@@ -32,7 +32,7 @@
 #endif
 
 #include "context-private.h"
-#include <libdivecomputer/custom_serial.h>
+#include <libdivecomputer/custom_io.h>
 
 struct dc_context_t {
 	dc_loglevel_t loglevel;
@@ -46,7 +46,8 @@ struct dc_context_t {
 	struct timeval timestamp;
 #endif
 #endif
-	dc_custom_serial_t *custom_serial;
+	dc_custom_io_t *custom_io;
+	dc_user_device_t *user_device;
 };
 
 #ifdef ENABLE_LOGGING
@@ -197,7 +198,7 @@ dc_context_new (dc_context_t **out)
 #endif
 #endif
 
-	context->custom_serial = NULL;
+	context->custom_io = NULL;
 
 	*out = context;
 
@@ -213,20 +214,21 @@ dc_context_free (dc_context_t *context)
 }
 
 dc_status_t
-dc_context_set_custom_serial (dc_context_t *context, dc_custom_serial_t *custom_serial)
+dc_context_set_custom_io (dc_context_t *context, dc_custom_io_t *custom_io, dc_user_device_t *user_device)
 {
 	if (context == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	context->custom_serial = custom_serial;
+	context->custom_io = custom_io;
+	custom_io->user_device = user_device;
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_custom_serial_t*
-_dc_context_custom_serial (dc_context_t *context)
+dc_custom_io_t*
+_dc_context_custom_io (dc_context_t *context)
 {
-	return context->custom_serial;
+	return context->custom_io;
 }
 
 dc_status_t
