@@ -27,6 +27,7 @@
 #include "context-private.h"
 #include "device-private.h"
 #include "ringbuffer.h"
+#include "rbstream.h"
 #include "array.h"
 
 #define VTABLE(abstract)	((oceanic_common_device_vtable_t *) abstract->vtable)
@@ -39,22 +40,6 @@
 
 #define INVALID 0
 
-static unsigned int
-ifloor (unsigned int x, unsigned int n)
-{
-	// Round down to next lower multiple.
-	return (x / n) * n;
-}
-
-
-static unsigned int
-iceil (unsigned int x, unsigned int n)
-{
-	// Round up to next higher multiple.
-	return ((x + n - 1) / n) * n;
-}
-
-
 static unsigned int
 get_profile_first (const unsigned char data[], const oceanic_common_layout_t *layout)
 {
@@ -234,177 +219,94 @@ oceanic_common_device_logbook (dc_device_t *abstract, dc_event_progress_t *progr
 		return DC_STATUS_DATAFORMAT;
 	}
 
-	// Convert the first/last pointers to begin/end/count pointers.
-	unsigned int rb_logbook_entry_begin, rb_logbook_entry_end,
-		rb_logbook_entry_size;
+	// Calculate the end pointer and the number of bytes.
+	unsigned int rb_logbook_end, rb_logbook_size;
 	if (layout->pt_mode_global == 0) {
-		rb_logbook_entry_begin = rb_logbook_first;
-		rb_logbook_entry_end   = RB_LOGBOOK_INCR (rb_logbook_last, layout->rb_logbook_entry_size, layout);
-		rb_logbook_entry_size  = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout) + layout->rb_logbook_entry_size;
+		rb_logbook_end  = RB_LOGBOOK_INCR (rb_logbook_last, layout->rb_logbook_entry_size, layout);
+		rb_logbook_size = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout) + layout->rb_logbook_entry_size;
 	} else {
-		rb_logbook_entry_begin = rb_logbook_first;
-		rb_logbook_entry_end   = rb_logbook_last;
-		rb_logbook_entry_size  = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout);
+		rb_logbook_end  = rb_logbook_last;
+		rb_logbook_size = RB_LOGBOOK_DISTANCE (rb_logbook_first, rb_logbook_last, layout);
 		// In a typical ringbuffer implementation with only two begin/end
 		// pointers, there is no distinction possible between an empty and
 		// a full ringbuffer. We always consider the ringbuffer full in
 		// that case, because an empty ringbuffer can be detected by
 		// inspecting the logbook entries once they are downloaded.
 		if (rb_logbook_first == rb_logbook_last)
-			rb_logbook_entry_size = layout->rb_logbook_end - layout->rb_logbook_begin;
+			rb_logbook_size = layout->rb_logbook_end - layout->rb_logbook_begin;
 	}
 
-	// Check whether the ringbuffer is full.
-	int full = (rb_logbook_entry_size == (layout->rb_logbook_end - layout->rb_logbook_begin));
-
-	// Align the pointers to page boundaries.
-	unsigned int rb_logbook_page_begin, rb_logbook_page_end,
-		rb_logbook_page_size;
-	if (full) {
-		// Full ringbuffer.
-		rb_logbook_page_begin = iceil (rb_logbook_entry_end, PAGESIZE);
-		rb_logbook_page_end   = rb_logbook_page_begin;
-		rb_logbook_page_size  = rb_logbook_entry_size;
-	} else {
-		// Non-full ringbuffer.
-		rb_logbook_page_begin = ifloor (rb_logbook_entry_begin, PAGESIZE);
-		rb_logbook_page_end   = iceil (rb_logbook_entry_end, PAGESIZE);
-		rb_logbook_page_size  = rb_logbook_entry_size +
-			(rb_logbook_entry_begin - rb_logbook_page_begin) +
-			(rb_logbook_page_end - rb_logbook_entry_end);
-	}
-
-	// Check whether the last entry is not aligned to a page boundary.
-	int unaligned = (rb_logbook_entry_end != rb_logbook_page_end);
-
 	// Update and emit a progress event.
 	progress->current += PAGESIZE;
 	progress->maximum += PAGESIZE;
-	progress->maximum -= (layout->rb_logbook_end - layout->rb_logbook_begin) - rb_logbook_page_size;
+	progress->maximum -= (layout->rb_logbook_end - layout->rb_logbook_begin) - rb_logbook_size;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
 
 	// Exit if there are no dives.
-	if (rb_logbook_page_size == 0) {
+	if (rb_logbook_size == 0) {
 		return DC_STATUS_SUCCESS;
 	}
 
 	// Allocate memory for the logbook entries.
-	if (!dc_buffer_resize (logbook, rb_logbook_page_size))
+	if (!dc_buffer_resize (logbook, rb_logbook_size))
 		return DC_STATUS_NOMEMORY;
 
 	// Cache the logbook pointer.
 	unsigned char *logbooks = dc_buffer_get_data (logbook);
 
-	// Since entries are not necessary aligned on page boundaries,
-	// the memory buffer may contain padding entries on both sides.
-	// The memory area which contains the valid entries is marked
-	// with a number of additional variables.
-	unsigned int begin = 0;
-	unsigned int end = rb_logbook_page_size;
-	if (!full) {
-		begin += rb_logbook_entry_begin - rb_logbook_page_begin;
-		end -= rb_logbook_page_end - rb_logbook_entry_end;
+	// Create the ringbuffer stream.
+	dc_rbstream_t *rbstream = NULL;
+	rc = dc_rbstream_new (&rbstream, abstract, PAGESIZE, PAGESIZE * device->multipage, layout->rb_logbook_begin, layout->rb_logbook_end, rb_logbook_end);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to create the ringbuffer stream.");
+		return rc;
 	}
 
 	// The logbook ringbuffer is read backwards to retrieve the most recent
 	// entries first. If an already downloaded entry is identified (by means
 	// of its fingerprint), the transfer is aborted immediately to reduce
-	// the transfer time. When necessary, padding entries are downloaded
-	// (but not processed) to align all read requests on page boundaries.
+	// the transfer time.
 	unsigned int nbytes = 0;
-	unsigned int current = end;
-	unsigned int offset = rb_logbook_page_size;
-	unsigned int address = rb_logbook_page_end;
-	while (nbytes < rb_logbook_page_size) {
-		// Handle the ringbuffer wrap point.
-		if (address == layout->rb_logbook_begin)
-			address = layout->rb_logbook_end;
+	unsigned int offset = rb_logbook_size;
+	while (nbytes < rb_logbook_size) {
+		// Move to the start of the current entry.
+		offset -= layout->rb_logbook_entry_size;
 
-		// Calculate the optimal packet size.
-		unsigned int len = PAGESIZE * device->multipage;
-		if (layout->rb_logbook_begin + len > address)
-			len = address - layout->rb_logbook_begin; // End of ringbuffer.
-		if (nbytes + len > rb_logbook_page_size)
-			len = rb_logbook_page_size - nbytes; // End of logbooks.
-
-		// Move to the start of the current page.
-		address -= len;
-		offset -= len;
-
-		// Read the logbook page.
-		rc = dc_device_read (abstract, address, logbooks + offset, len);
+		// Read the logbook entry.
+		rc = dc_rbstream_read (rbstream, progress, logbooks + offset, layout->rb_logbook_entry_size);
 		if (rc != DC_STATUS_SUCCESS) {
-			ERROR (abstract->context, "Failed to read the memory page.");
+			ERROR (abstract->context, "Failed to read the memory.");
+			dc_rbstream_free (rbstream);
 			return rc;
 		}
 
-		// Update and emit a progress event.
-		progress->current += len;
-		device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
+		nbytes += layout->rb_logbook_entry_size;
 
-		// A full ringbuffer needs some special treatment to avoid
-		// having to download the first/last page twice. When a full
-		// ringbuffer is not aligned to page boundaries, this page
-		// will contain both the most recent and oldest entry.
-		if (full && unaligned) {
-			if (nbytes == 0) {
-				// After downloading the first page, move both the oldest
-				// and most recent entries to their correct location.
-				unsigned int oldest = rb_logbook_page_end - rb_logbook_entry_end;
-				unsigned int newest  = len - oldest;
-				// Move the oldest entries down to the start of the buffer.
-				memcpy (logbooks, logbooks + offset + newest, oldest);
-				// Move the newest entries up to the end of the buffer.
-				memmove (logbooks + offset + oldest, logbooks + offset, newest);
-				// Adjust the current page offset to the new position.
-				offset += oldest;
-			} else if (nbytes + len == rb_logbook_page_size) {
-				// After downloading the last page, pretend we have also
-				// downloaded those oldest entries from the first page.
-				offset = 0;
-			}
-		}
-
-		nbytes += len;
-
-		// Process the logbook entries.
-		int abort = 0;
-		while (current >= offset + layout->rb_logbook_entry_size &&
-			current != offset && current != begin)
-		{
-			// Move to the start of the current entry.
-			current -= layout->rb_logbook_entry_size;
-
-			// Check for uninitialized entries. Normally, such entries are
-			// never present, except when the ringbuffer is actually empty,
-			// but the ringbuffer pointers are not set to their empty values.
-			// This appears to happen on some devices, and we attempt to
-			// fix this here.
-			if (array_isequal (logbooks + current, layout->rb_logbook_entry_size, 0xFF)) {
-				WARNING (abstract->context, "Uninitialized logbook entries detected!");
-				begin = current + layout->rb_logbook_entry_size;
-				abort = 1;
-				break;
-			}
-
-			// Compare the fingerprint to identify previously downloaded entries.
-			if (memcmp (logbooks + current, device->fingerprint, layout->rb_logbook_entry_size) == 0) {
-				begin = current + layout->rb_logbook_entry_size;
-				abort = 1;
-				break;
-			}
-		}
-
-		// Stop reading pages too.
-		if (abort)
+		// Check for uninitialized entries. Normally, such entries are
+		// never present, except when the ringbuffer is actually empty,
+		// but the ringbuffer pointers are not set to their empty values.
+		// This appears to happen on some devices, and we attempt to
+		// fix this here.
+		if (array_isequal (logbooks + offset, layout->rb_logbook_entry_size, 0xFF)) {
+			WARNING (abstract->context, "Uninitialized logbook entries detected!");
+			offset += layout->rb_logbook_entry_size;
 			break;
+		}
+
+		// Compare the fingerprint to identify previously downloaded entries.
+		if (memcmp (logbooks + offset, device->fingerprint, layout->rb_logbook_entry_size) == 0) {
+			offset += layout->rb_logbook_entry_size;
+			break;
+		}
 	}
 
 	// Update and emit a progress event.
-	progress->maximum -= rb_logbook_page_size - nbytes;
+	progress->maximum -= rb_logbook_size - nbytes;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
 
-	dc_buffer_slice (logbook, begin, end - begin);
+	dc_buffer_slice (logbook, offset, rb_logbook_size - offset);
+
+	dc_rbstream_free (rbstream);
 
 	return DC_STATUS_SUCCESS;
 }
@@ -492,20 +394,24 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 	progress->maximum -= (layout->rb_profile_end - layout->rb_profile_begin) - rb_profile_size;
 	device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
 
+	// Create the ringbuffer stream.
+	dc_rbstream_t *rbstream = NULL;
+	rc = dc_rbstream_new (&rbstream, abstract, PAGESIZE, PAGESIZE * device->multipage, layout->rb_profile_begin, layout->rb_profile_end, rb_profile_end);
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to create the ringbuffer stream.");
+		return rc;
+	}
+
 	// Memory buffer for the profile data.
 	unsigned char *profiles = (unsigned char *) malloc (rb_profile_size + rb_logbook_size);
 	if (profiles == NULL) {
+		ERROR (abstract->context, "Failed to allocate memory.");
+		dc_rbstream_free (rbstream);
 		return DC_STATUS_NOMEMORY;
 	}
 
 	// Keep track of the current position.
 	unsigned int offset = rb_profile_size + rb_logbook_size;
-	unsigned int address = rb_profile_end;
-
-	// When using multipage reads, the last packet can contain data from more
-	// than one dive. Therefore, the remaining data of this package (and its
-	// size) needs to be preserved for the next dive.
-	unsigned int available = 0;
 
 	// Traverse the logbook ringbuffer backwards to retrieve the most recent
 	// dives first. The logbook ringbuffer is linearized at this point, so
@@ -528,6 +434,7 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 		{
 			ERROR (abstract->context, "Invalid ringbuffer pointer detected (0x%06x 0x%06x).",
 				rb_entry_first, rb_entry_last);
+			dc_rbstream_free (rbstream);
 			free (profiles);
 			return DC_STATUS_DATAFORMAT;
 		}
@@ -549,56 +456,33 @@ oceanic_common_device_profile (dc_device_t *abstract, dc_event_progress_t *progr
 			break;
 		}
 
-		// Read the profile data.
-		unsigned int nbytes = available;
-		while (nbytes < rb_entry_size + gap) {
-			// Handle the ringbuffer wrap point.
-			if (address == layout->rb_profile_begin)
-				address = layout->rb_profile_end;
+		// Move to the start of the current dive.
+		offset -= rb_entry_size + gap;
 
-			// Calculate the optimal packet size.
-			unsigned int len = PAGESIZE * device->multipage;
-			if (layout->rb_profile_begin + len > address)
-				len = address - layout->rb_profile_begin; // End of ringbuffer.
-			if (nbytes + len > remaining)
-				len = remaining - nbytes; // End of profile.
-
-			// Move to the start of the current page.
-			address -= len;
-			offset -= len;
-
-			// Read the profile page.
-			rc = dc_device_read (abstract, address, profiles + offset, len);
-			if (rc != DC_STATUS_SUCCESS) {
-				free (profiles);
-				return rc;
-			}
-
-			// Update and emit a progress event.
-			progress->current += len;
-			device_event_emit (abstract, DC_EVENT_PROGRESS, progress);
-
-			nbytes += len;
+		// Read the dive.
+		rc = dc_rbstream_read (rbstream, progress, profiles + offset, rb_entry_size + gap);
+		if (rc != DC_STATUS_SUCCESS) {
+			ERROR (abstract->context, "Failed to read the dive.");
+			dc_rbstream_free (rbstream);
+			free (profiles);
+			return rc;
 		}
 
-		available = nbytes - (rb_entry_size + gap);
 		remaining -= rb_entry_size + gap;
 		previous = rb_entry_first;
 
 		// Prepend the logbook entry to the profile data. The memory buffer is
-		// large enough to store this entry, but any data that belongs to the
-		// next dive needs to be moved down first.
-		if (available)
-			memmove (profiles + offset - layout->rb_logbook_entry_size, profiles + offset, available);
+		// large enough to store this entry.
 		offset -= layout->rb_logbook_entry_size;
-		memcpy (profiles + offset + available, logbooks + entry, layout->rb_logbook_entry_size);
+		memcpy (profiles + offset, logbooks + entry, layout->rb_logbook_entry_size);
 
-		unsigned char *p = profiles + offset + available;
+		unsigned char *p = profiles + offset;
 		if (callback && !callback (p, rb_entry_size + layout->rb_logbook_entry_size, p, layout->rb_logbook_entry_size, userdata)) {
 			break;
 		}
 	}
 
+	dc_rbstream_free (rbstream);
 	free (profiles);
 
 	return DC_STATUS_SUCCESS;
