@@ -22,15 +22,17 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdarg.h>
 
 #ifdef _MSC_VER
 #define snprintf _snprintf
 #endif
 
-#include <libdivecomputer/shearwater_predator.h>
-#include <libdivecomputer/shearwater_petrel.h>
 #include <libdivecomputer/units.h>
 
+#include "shearwater_predator.h"
+#include "shearwater_petrel.h"
+#include "shearwater_common.h"
 #include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
@@ -53,12 +55,13 @@
 #define IMPERIAL 1
 
 #define NGASMIXES 10
+#define MAXSTRINGS 32
 
 typedef struct shearwater_predator_parser_t shearwater_predator_parser_t;
 
 struct shearwater_predator_parser_t {
 	dc_parser_t base;
-	unsigned int petrel;
+	unsigned int model;
 	unsigned int samplesize;
 	// Cached fields.
 	unsigned int cached;
@@ -67,8 +70,13 @@ struct shearwater_predator_parser_t {
 	unsigned int ngasmixes;
 	unsigned int oxygen[NGASMIXES];
 	unsigned int helium[NGASMIXES];
+	double calibration[3];
 	unsigned int serial;
 	dc_divemode_t mode;
+	unsigned char logversion;
+
+	/* String fields */
+	dc_field_string_t strings[MAXSTRINGS];
 };
 
 static dc_status_t shearwater_predator_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
@@ -111,8 +119,8 @@ shearwater_predator_find_gasmix (shearwater_predator_parser_t *parser, unsigned
 }
 
 
-static dc_status_t
-shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial, unsigned int petrel)
+dc_status_t
+shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model, unsigned int serial)
 {
 	shearwater_predator_parser_t *parser = NULL;
 	const dc_parser_vtable_t *vtable = NULL;
@@ -121,7 +129,7 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 	if (out == NULL)
 		return DC_STATUS_INVALIDARGS;
 
-	if (petrel) {
+	if (model != PREDATOR) {
 		vtable = &shearwater_petrel_parser_vtable;
 		samplesize = SZ_SAMPLE_PETREL;
 	} else {
@@ -136,14 +144,10 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 		return DC_STATUS_NOMEMORY;
 	}
 
-	// Initialize the base class.
-	parser->petrel = petrel;
+	// Set the default values.
+	parser->model = model;
+	parser->samplesize = samplesize;
 	parser->serial = serial;
-	if (petrel) {
-		parser->samplesize = SZ_SAMPLE_PETREL;
-	} else {
-		parser->samplesize = SZ_SAMPLE_PREDATOR;
-	}
 
 	// Set the default values.
 	parser->cached = 0;
@@ -162,20 +166,6 @@ shearwater_common_parser_create (dc_parser_t **out, dc_context_t *context, unsig
 }
 
 
-dc_status_t
-shearwater_predator_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial)
-{
-	return shearwater_common_parser_create (out, context, serial, 0);
-}
-
-
-dc_status_t
-shearwater_petrel_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int serial)
-{
-	return shearwater_common_parser_create (out, context, serial, 1);
-}
-
-
 static dc_status_t
 shearwater_predator_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
@@ -210,11 +200,151 @@ shearwater_predator_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *d
 	if (!dc_datetime_gmtime (datetime, ticks))
 		return DC_STATUS_DATAFORMAT;
 
+	datetime->timezone = DC_TIMEZONE_NONE;
+
 	return DC_STATUS_SUCCESS;
 }
 
+/*
+ * These string cache interfaces should be some generic
+ * library rather than copied for all the dive computers.
+ *
+ * This is just copied from the EON Steel code.
+ */
+static void
+add_string(shearwater_predator_parser_t *parser, const char *desc, const char *value)
+{
+	int i;
 
-#define BUFLEN 32
+	for (i = 0; i < MAXSTRINGS; i++) {
+		dc_field_string_t *str = parser->strings+i;
+		if (str->desc)
+			continue;
+		str->desc = desc;
+		str->value = strdup(value);
+		break;
+	}
+}
+
+static void
+add_string_fmt(shearwater_predator_parser_t *parser, const char *desc, const char *fmt, ...)
+{
+	char buffer[256];
+	va_list ap;
+
+	/*
+	 * We ignore the return value from vsnprintf, and we
+	 * always NUL-terminate the destination buffer ourselves.
+	 *
+	 * That way we don't have to worry about random bad legacy
+	 * implementations.
+	 */
+	va_start(ap, fmt);
+	buffer[sizeof(buffer)-1] = 0;
+	(void) vsnprintf(buffer, sizeof(buffer)-1, fmt, ap);
+	va_end(ap);
+
+	return add_string(parser, desc, buffer);
+}
+
+// The Battery state is a big-endian word:
+//
+//  ffff = not paired / no comms for 90 s
+//  fffe = no comms for 30 s
+//
+// Otherwise:
+//   - top four bits are battery state (0 - normal, 1 - critical, 2 - warning)
+//   - bottom 12 bits are pressure in 2 psi increments (0..8k psi)
+//
+// This returns the state as a bitmask (so you can see all states it had
+// during the dive). Note that we currently do not report pairing and
+// communication lapses. Todo?
+static unsigned int
+battery_state(const unsigned char *data)
+{
+	unsigned int pressure = array_uint16_be(data);
+	unsigned int state;
+
+	if ((pressure & 0xFFF0) == 0xFFF0)
+		return 0;
+	state = pressure >> 12;
+	if (state > 2)
+		return 0;
+	return 1u << state;
+}
+
+// Show the battery state
+//
+// NOTE! Right now it only shows the most serious bit
+// but the code is set up so that we could perhaps
+// indicate that the battery is on the edge (ie it
+// reported both "normal" _and_ "warning" during the
+// dive - maybe that would be a "starting to warn")
+//
+// We could also report unpaired and comm errors.
+static void
+add_battery_info(shearwater_predator_parser_t *parser, const char *desc, unsigned int state)
+{
+	if (state >= 1 && state <= 7) {
+		static const char *states[8] = {
+			"",		// 000 - No state bits, not used
+			"normal",	// 001 - only normal
+			"critical",	// 010 - only critical
+			"critical",	// 011 - both normal and critical
+			"warning",	// 100 - only warning
+			"warning",	// 101 - normal and warning
+			"critical",	// 110 - warning and critical
+			"critical",	// 111 - normal, warning and critical
+		};
+		add_string(parser, desc, states[state]);
+	}
+}
+
+static void
+add_deco_model(shearwater_predator_parser_t *parser, const unsigned char *data)
+{
+	switch	(data[67]) {
+	case 0:
+		add_string_fmt(parser, "Deco model", "GF %u/%u", data[4], data[5]);
+		break;
+	case 1:
+		add_string_fmt(parser, "Deco model", "VPM-B +%u", data[68]);
+		break;
+	case 2:
+		add_string_fmt(parser, "Deco model", "VPM-B/GFS +%u %u%%", data[68], data[85]);
+		break;
+	default:
+		add_string_fmt(parser, "Deco model", "Unknown model %d", data[67]);
+	}
+}
+
+static void
+add_battery_type(shearwater_predator_parser_t *parser, const unsigned char *data)
+{
+	if (parser->logversion < 7)
+		return;
+
+	switch (data[120]) {
+	case 1:
+		add_string(parser, "Battery type", "1.5V Alkaline");
+		break;
+	case 2:
+		add_string(parser, "Battery type", "1.5V Lithium");
+		break;
+	case 3:
+		add_string(parser, "Battery type", "1.2V NiMH");
+		break;
+	case 4:
+		add_string(parser, "Battery type", "3.6V Saft");
+		break;
+	case 5:
+		add_string(parser, "Battery type", "3.7V Li-Ion");
+		break;
+	default:
+		add_string_fmt(parser, "Battery type", "unknown type %d", data[120]);
+		break;
+	}
+}
 
 static dc_status_t
 shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
@@ -234,8 +364,17 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 		return DC_STATUS_DATAFORMAT;
 	}
 
+	// Log versions before 6 weren't reliably stored in the data, but
+	// 6 is also the oldest version that we assume in our code
+	parser->logversion = 6;
+	if (data[127] > 6)
+		parser->logversion = data[127];
+	INFO(abstract->context, "Shearwater log version %u\n", parser->logversion);
+
+	memset(parser->strings, 0, sizeof(parser->strings));
+
 	// Adjust the footersize for the final block.
-	if (parser->petrel || array_uint16_be (data + size - footersize) == 0xFFFD) {
+	if (parser->model > PREDATOR || array_uint16_be (data + size - footersize) == 0xFFFD) {
 		footersize += SZ_BLOCK;
 		if (size < headersize + footersize) {
 			ERROR (abstract->context, "Invalid data length.");
@@ -252,6 +391,9 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 	unsigned int helium[NGASMIXES] = {0};
 	unsigned int o2_previous = 0, he_previous = 0;
 
+	// Transmitter battery levels
+	unsigned int t1_battery = 0, t2_battery = 0;
+
 	unsigned int offset = headersize;
 	unsigned int length = size - footersize;
 	while (offset < length) {
@@ -294,9 +436,30 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 			he_previous = he;
 		}
 
+		// Transmitter battery levels
+		if (parser->logversion >= 7) {
+			// T1 at offset 27, T2 at offset 19
+			t1_battery |= battery_state(data + offset + 27);
+			t2_battery |= battery_state(data + offset + 19);
+		}
+
 		offset += parser->samplesize;
 	}
 
+	// Cache sensor calibration for later use
+	parser->calibration[0] = array_uint16_be(data + 87) / 100000.0;
+	parser->calibration[1] = array_uint16_be(data + 89) / 100000.0;
+	parser->calibration[2] = array_uint16_be(data + 91) / 100000.0;
+	// The Predator expects the mV output of the cells to be within 30mV
+	// to 70mV in 100% O2 at 1 atmosphere.
+	// If the calibration value is scaled with a factor 2.2, then the
+	// sensors lines up and matches the average.
+	if (parser->model == PREDATOR) {
+		for (size_t i = 0; i < 3; ++i) {
+			parser->calibration[i] *= 2.2;
+		}
+	}
+
 	// Cache the data for later use.
 	parser->headersize = headersize;
 	parser->footersize = footersize;
@@ -306,6 +469,13 @@ shearwater_predator_parser_cache (shearwater_predator_parser_t *parser)
 		parser->helium[i] = helium[i];
 	}
 	parser->mode = mode;
+	add_string_fmt(parser, "Serial", "%08x", parser->serial);
+	add_string_fmt(parser, "FW Version", "%2x", data[19]);
+	add_deco_model(parser, data);
+	add_battery_type(parser, data);
+	add_string_fmt(parser, "Battery at end", "%.1f V", data[9] / 10.0);
+	add_battery_info(parser, "T1 battery", t1_battery);
+	add_battery_info(parser, "T2 battery", t2_battery);
 	parser->cached = 1;
 
 	return DC_STATUS_SUCCESS;
@@ -334,7 +504,6 @@ shearwater_predator_parser_get_field (dc_parser_t *abstract, dc_field_type_t typ
 	dc_salinity_t *water = (dc_salinity_t *) value;
 	dc_field_string_t *string = (dc_field_string_t *) value;
 	unsigned int density = 0;
-	char buf[BUFLEN];
 
 	if (value) {
 		switch (type) {
@@ -370,56 +539,14 @@ shearwater_predator_parser_get_field (dc_parser_t *abstract, dc_field_type_t typ
 			*((dc_divemode_t *) value) = parser->mode;
 			break;
 		case DC_FIELD_STRING:
-			switch(flags) {
-			case 0: // Battery
-				string->desc = "Battery at end";
-				snprintf(buf, BUFLEN, "%.1f", data[9] / 10.0);
-				break;
-			case 1: // Serial
-				string->desc = "Serial";
-				snprintf(buf, BUFLEN, "%08x", parser->serial);
-				break;
-			case 2: // FW Version
-				string->desc = "FW Version";
-				snprintf(buf, BUFLEN, "%2x", data[19]);
-				break;
-			case 3: /* Deco model */
-				string->desc = "Deco model";
-				switch (data[67]) {
-				case 0:
-					strncpy(buf, "GF", BUFLEN);
+			if (flags < MAXSTRINGS) {
+				dc_field_string_t *p = parser->strings + flags;
+				if (p->desc) {
+					*string = *p;
 					break;
-				case 1:
-					strncpy(buf, "VPM-B", BUFLEN);
-					break;
-				case 2:
-					strncpy(buf, "VPM-B/GFS", BUFLEN);
-					break;
-				default:
-					return DC_STATUS_DATAFORMAT;
 				}
-				break;
-			case 4: /* Deco model info */
-				string->desc = "Deco model info";
-				switch (data[67]) {
-				case 0:
-					snprintf(buf, BUFLEN, "GF %u/%u", data[4], data[5]);
-					break;
-				case 1:
-					snprintf(buf, BUFLEN, "VPM-B +%u", data[68]);
-					break;
-				case 2:
-					snprintf(buf, BUFLEN, "VPM-B/GFS +%u %u%%", data[68], data[85]);
-					break;
-				default:
-					return DC_STATUS_DATAFORMAT;
-				}
-				break;
-			default:
-				return DC_STATUS_UNSUPPORTED;
 			}
-			string->value = strdup(buf);
-			break;
+			return DC_STATUS_UNSUPPORTED;
 		default:
 			return DC_STATUS_UNSUPPORTED;
 		}
@@ -451,6 +578,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 	unsigned int time = 0;
 	unsigned int offset = parser->headersize;
 	unsigned int length = size - parser->footersize;
+
 	while (offset < length) {
 		dc_sample_value_t sample = {0};
 
@@ -492,12 +620,25 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 		unsigned int status = data[offset + 11];
 
 		if ((status & OC) == 0) {
-			// PPO2 -- only return PPO2 if we are in closed circuit mode
-			sample.ppo2 = data[offset + 6] / 100.0;
-			if (callback) callback (DC_SAMPLE_PPO2, sample, userdata);
+			// PPO2
+			if ((status & PPO2_EXTERNAL) == 0) {
+#ifdef SENSOR_AVERAGE
+				sample.ppo2 = data[offset + 6] / 100.0;
+				if (callback) callback (DC_SAMPLE_PPO2, sample, userdata);
+#else
+				sample.ppo2 = data[offset + 12] * parser->calibration[0];
+				if (callback && (data[86] & 0x01)) callback (DC_SAMPLE_PPO2, sample, userdata);
+
+				sample.ppo2 = data[offset + 14] * parser->calibration[1];
+				if (callback && (data[86] & 0x02)) callback (DC_SAMPLE_PPO2, sample, userdata);
+
+				sample.ppo2 = data[offset + 15] * parser->calibration[2];
+				if (callback && (data[86] & 0x04)) callback (DC_SAMPLE_PPO2, sample, userdata);
+#endif
+			}
 
 			// Setpoint
-			if (parser->petrel) {
+			if (parser->model > PREDATOR) {
 				sample.setpoint = data[offset + 18] / 100.0;
 			} else {
 				if (status & SETPOINT_HIGH) {
@@ -510,7 +651,7 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 		}
 
 		// CNS
-		if (parser->petrel) {
+		if (parser->model > PREDATOR) {
 			sample.cns = data[offset + 22] / 100.0;
 			if (callback) callback (DC_SAMPLE_CNS, sample, userdata);
 		}
@@ -546,8 +687,36 @@ shearwater_predator_parser_samples_foreach (dc_parser_t *abstract, dc_sample_cal
 		sample.deco.time = data[offset + 9] * 60;
 		if (callback) callback (DC_SAMPLE_DECO, sample, userdata);
 
+		// for logversion 7 and newer (introduced for Perdix AI)
+		// detect tank pressure
+		if (parser->logversion >= 7) {
+			// Pressure (2 psi).
+			// 0xFFFF is not paired / no coms for 90 seconds
+			// 0xFFFE no coms for 30 seconds
+			// top 4 bits battery level:
+			// 0 - normal, 1 - critical, 2 - warning
+			unsigned int pressure = array_uint16_be (data + offset + 27);
+			if ((pressure & 0xFFF0) != 0xFFF0) {
+				pressure &= 0x0FFF;
+				sample.pressure.tank = 0;
+				sample.pressure.value = pressure * 2 * PSI / BAR;
+				if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+			}
+			pressure = array_uint16_be (data + offset + 19);
+			if ((pressure & 0xFFF0) != 0xFFF0) {
+				pressure &= 0x0FFF;
+				sample.pressure.tank = 1;
+				sample.pressure.value = pressure * 2 * PSI / BAR;
+				if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
+			}
+			// Gas time remaining in minutes
+			if (data[offset + 21] < 0xFBu) {
+				sample.rbt = data[offset + 21];
+				if (callback) callback (DC_SAMPLE_RBT, sample, userdata);
+			}
+		}
+
 		offset += parser->samplesize;
 	}
-
 	return DC_STATUS_SUCCESS;
 }
