@@ -35,19 +35,20 @@
 
 #define NBITS 8
 
-#define SMARTPRO      0x10
-#define GALILEO       0x11
-#define ALADINTEC     0x12
-#define ALADINTEC2G   0x13
-#define SMARTCOM      0x14
-#define ALADIN2G      0x15
-#define SMARTTEC      0x18
-#define GALILEOTRIMIX 0x19
-#define SMARTZ        0x1C
-#define MERIDIAN      0x20
-#define CHROMIS       0x24
-#define MANTIS2       0x26
-#define G2            0x32
+#define SMARTPRO          0x10
+#define GALILEO           0x11
+#define ALADINTEC         0x12
+#define ALADINTEC2G       0x13
+#define SMARTCOM          0x14
+#define ALADIN2G          0x15
+#define SMARTTEC          0x18
+#define GALILEOTRIMIX     0x19
+#define SMARTZ            0x1C
+#define MERIDIAN          0x20
+#define CHROMIS           0x24
+#define MANTIS2           0x26
+#define G2                0x32
+#define ALADINSPORTMATRIX 0xa5
 
 #define UNSUPPORTED 0xFFFFFFFF
 
@@ -64,6 +65,8 @@
 #define GAUGE     0x00001000
 #define SALINITY  0x00100000
 
+#define EPOCH 946684800 // 2000-01-01 00:00:00 UTC
+
 typedef enum {
 	PRESSURE_DEPTH,
 	RBT,
@@ -519,7 +522,7 @@ uwatec_smart_parser_cache (uwatec_smart_parser_t *parser)
 				if (parser->model == GALILEO || parser->model == GALILEOTRIMIX ||
 					parser->model == ALADIN2G || parser->model == MERIDIAN ||
 					parser->model == CHROMIS || parser->model == MANTIS2 ||
-					parser->model == G2) {
+					parser->model == G2 || parser->model == ALADINSPORTMATRIX) {
 					unsigned int offset = header->tankpressure + 2 * i;
 					endpressure   = array_uint16_le(data + offset);
 					beginpressure = array_uint16_le(data + offset + 2 * header->ngases);
@@ -609,6 +612,7 @@ uwatec_smart_parser_create (dc_parser_t **out, dc_context_t *context, unsigned i
 		parser->nevents[2] = C_ARRAY_SIZE (uwatec_smart_galileo_events_2);
 		break;
 	case G2:
+	case ALADINSPORTMATRIX:
 		parser->headersize = 84;
 		parser->header = &uwatec_smart_trimix_header;
 		parser->samples = uwatec_smart_galileo_samples;
@@ -723,7 +727,7 @@ uwatec_smart_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime
 
 	unsigned int timestamp = array_uint32_le (abstract->data + 8);
 
-	dc_ticks_t ticks = parser->systime - (parser->devtime - timestamp) / 2;
+	dc_ticks_t ticks = EPOCH + timestamp / 2;
 
 	if (table->timezone != UNSUPPORTED) {
 		// For devices with timezone support, the UTC offset of the
@@ -948,7 +952,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 		if (parser->model == GALILEO || parser->model == GALILEOTRIMIX ||
 			parser->model == ALADIN2G || parser->model == MERIDIAN ||
 			parser->model == CHROMIS || parser->model == MANTIS2 ||
-			parser->model == G2) {
+			parser->model == G2 || parser->model == ALADINSPORTMATRIX) {
 			// Uwatec Galileo
 			id = uwatec_galileo_identify (data[offset]);
 		} else {
